package br.com.yupistudios.biopassid_flutter

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import br.com.yupistudios.biopassid_flutter.utils.BioPassIdUtils
import com.biopassid.biopassidsdk.BioPassIDEvent
import com.biopassid.biopassidsdk.CallBackBioPassID
import com.biopassid.biopassidsdk.Camera2BasicFragment
import com.biopassid.biopassidsdk.CameraActivity
import com.biopassid.biopassidsdk.bioPassIDConfigs.ConfigTheme

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

/** BiopassidFlutterPlugin */
class BiopassidFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    EventChannel.StreamHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var methodChannel: MethodChannel
    private lateinit var eventChannel: EventChannel
    private var eventSink: EventChannel.EventSink? = null
    private var activity: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel = MethodChannel(flutterPluginBinding.binaryMessenger, "biopassid_flutter")
        methodChannel.setMethodCallHandler(this)

        eventChannel =
            EventChannel(flutterPluginBinding.binaryMessenger, "biopassid_flutter/event_channel")
        eventChannel.setStreamHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getBioPassId") {
            try {
                val type = call.argument<String>("type")
                val biopassidConfig = call.argument<Map<String, Any>>("biopassidConfig")

                val config = BioPassIdUtils.mapToBioPassIdConfig(type!!, biopassidConfig!!)
                ConfigTheme.setConfigBioPass(config)

                Camera2BasicFragment.setCallbackBioPass(activity, object : CallBackBioPassID {
                    override fun receiveFrames(event: BioPassIDEvent) {
                        if (type != "LivenessDetectionBioPassIDConfig") {
                            CameraActivity.encerrarActivity()
                        }
                        val bioPassIdEvent = BioPassIdUtils.bioPassIdEventToMap(event)
                        eventSink?.success(bioPassIdEvent)
                    }
                })

                val intent = Intent(activity, CameraActivity::class.java)
                activity?.startActivity(intent)
                result.success(true)
            } catch (e: Exception) {
                result.error("UNAVAILABLE", e.message, null)
            }
            return
        }

        result.notImplemented()
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        this.activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivity() {
        this.activity = null
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        eventSink = events
    }

    override fun onCancel(arguments: Any?) {
        eventSink = null
    }
}
