package br.com.yupistudios.biopassid_flutter.utils

import android.util.Size
import com.biopassid.biopassidsdk.BioPassIDEvent
import com.biopassid.biopassidsdk.CameraPreset
import com.biopassid.biopassidsdk.bioPassIDConfigs.*
import com.biopassid.biopassidsdk.biopassapi.response.icao.ICAOResponse
import com.biopassid.biopassidsdk.biopassapi.response.icao.RequirementsICAOResponse
import com.biopassid.biopassidsdk.biopassapi.response.icao.ResultCheckICAOResponse
import com.biopassid.biopassidsdk.biopassapi.response.spoof.FaceSpoofResponse

class BioPassIdUtils {
    companion object {
        fun mapToBioPassIdConfig(type: String, biopassidConfig: Map<String, Any>): BioPassIDConfig {
            val cameraPreset: Size = when (biopassidConfig["cameraPreset"] as String) {
                "LIVENESS" -> CameraPreset.LIVENESS
                else -> CameraPreset.FACE_FRONT
            }

            val defaultCameraPosition: CameraLensDirection = when (biopassidConfig["defaultCameraPosition"] as String) {
                "BACK" -> CameraLensDirection.BACK
                else -> CameraLensDirection.FRONT
            }

            val apiKey = biopassidConfig["apiKey"] as Map<String, String>
            val multibiometricsApiKey = apiKey["multibiometrics"]!!
            val qualityApiKey = apiKey["quality"]!!

            val showCaptureButton = biopassidConfig["showCaptureButton"] as Boolean
            val showFlashButton = biopassidConfig["showFlashButton"] as Boolean
            val showFlipCameraButton =
                biopassidConfig["showFlipCameraButton"] as Boolean
            val flashEnabledByDefault =
                biopassidConfig["flashEnabledByDefault"] as Boolean
            val icao = biopassidConfig["icao"] as Boolean
            val spoofing = biopassidConfig["spoofing"] as Boolean
            val showHelpText = biopassidConfig["showHelpText"] as Boolean

            val strings = biopassidConfig["strings"] as Map<String, String>
//            val chooseOtherButton = strings["chooseOtherButton"]!!
//            val confirmButton = strings["confirmButton"]!!
//            val sendPhoto = strings["sendPhoto"]!!
            val screenTitle = strings["screenTitle"]!!
            val helpText = strings["helpText"]!!
//            val newStrings = BioPassIDStrings(chooseOtherButton, confirmButton, sendPhoto, screenTitle, helpText)
            val newStrings = BioPassIDStrings(screenTitle, helpText)

            val styles = biopassidConfig["styles"] as Map<String, String>
            // val captureButtonColor = styles["captureButtonColor"]!!
            // val flipCameraColor = styles["flipCameraColor"]!!
            val faceColor = styles["faceColor"]!!
            val overlayColor = styles["overlayColor"]!!
            val flashButtonEnabledColor = styles["flashButtonEnabledColor"]!!
//             val newStyles = BioPassIDStyles(
//                 captureButtonColor,
//                 flipCameraColor,
//                 faceColor,
//                 overlayColor,
//                 flashButtonEnabledColor
//             )
            val newStyles = BioPassIDStyles(
                faceColor,
                overlayColor,
                flashButtonEnabledColor
            )

            val config: BioPassIDConfig = when (type) {
                "LivenessDetectionBioPassIDConfig" -> BioPassIDConfigPreset.getConfig(BioPassIDConfigType.LIVENESS_DETECTION)
                else -> BioPassIDConfigPreset.getConfig(BioPassIDConfigType.FACE_DETECTION)
            }

            if (multibiometricsApiKey != "") {
                config.setMultibiometricsKey(multibiometricsApiKey)
            }
            if (qualityApiKey != "") {
                config.setQualityKey(qualityApiKey)
            }

            config.setDefaultCameraPosition(defaultCameraPosition)
                .setCameraPreset(cameraPreset)
                .setStrings(newStrings).setStyles(newStyles)
                .setShowFlipCameraButton(showFlipCameraButton)
                .setFlashEnabledByDefault(flashEnabledByDefault).setShowFlashButton(showFlashButton)
                .setShowCaptureButton(showCaptureButton).setICAO(icao).setSpoofing(spoofing)
                .setShowHelpText(showHelpText)

            return config
        }

        fun bioPassIdEventToMap(event: BioPassIDEvent): Map<String, Any?> {
            return hashMapOf(
                "photo" to event.photo,
                "icao" to if (event.icao != null) icaoResponseToMap(event.icao) else null,
                "spoof" to if (event.spoof != null) faceSpoofResponseToMap(event.spoof) else null
            )
        }

        private fun icaoResponseToMap(icaoResponse: ICAOResponse?): Map<String, Any?> {
            return hashMapOf(
                "faceImageConformed" to icaoResponse?.faceImageConformed,
                "isAllConformed" to icaoResponse?.isAllConformed,
                "requirements" to if (icaoResponse?.requirements != null) requirementsICAOResponseToMap(
                    icaoResponse.requirements
                ) else null
            )
        }

        private fun requirementsICAOResponseToMap(requirementsICAOResponse: RequirementsICAOResponse?): Map<String, Any?> {
            return hashMapOf(
                "blurred" to resultCheckICAOResponseToMap(requirementsICAOResponse?.blurred),
                "lookingAway" to resultCheckICAOResponseToMap(requirementsICAOResponse?.lookingAway),
                "inkMarked" to resultCheckICAOResponseToMap(requirementsICAOResponse?.inkMarked),
                "unnaturalSkin" to resultCheckICAOResponseToMap(requirementsICAOResponse?.unnaturalSkin),
                "tooDarkLight" to resultCheckICAOResponseToMap(requirementsICAOResponse?.tooDarkLight),
                "washedOut" to resultCheckICAOResponseToMap(requirementsICAOResponse?.washedOut),
                "pixelation" to resultCheckICAOResponseToMap(requirementsICAOResponse?.pixelation),
                "hairAcrossEyes" to resultCheckICAOResponseToMap(requirementsICAOResponse?.hairAcrossEyes),
                "eyesClosed" to resultCheckICAOResponseToMap(requirementsICAOResponse?.eyesClosed),
                "variedBackground" to resultCheckICAOResponseToMap(requirementsICAOResponse?.variedBackground),
                "roll_Pitch_Yaw" to resultCheckICAOResponseToMap(requirementsICAOResponse?.roll_Pitch_Yaw),
                "flashReflectionOnSkin" to resultCheckICAOResponseToMap(requirementsICAOResponse?.flashReflectionOnSkin),
                "redEyes" to resultCheckICAOResponseToMap(requirementsICAOResponse?.redEyes),
                "shadowsBehindHead" to resultCheckICAOResponseToMap(requirementsICAOResponse?.shadowsBehindHead),
                "shadowsAcrossFace" to resultCheckICAOResponseToMap(requirementsICAOResponse?.shadowsAcrossFace),
                "darkTintedLenses" to resultCheckICAOResponseToMap(requirementsICAOResponse?.darkTintedLenses),
                "flashReflectionOnLenses" to resultCheckICAOResponseToMap(requirementsICAOResponse?.flashReflectionOnLenses),
                "framesTooHeavy" to resultCheckICAOResponseToMap(requirementsICAOResponse?.framesTooHeavy),
                "framesCoveringEyes" to resultCheckICAOResponseToMap(requirementsICAOResponse?.framesCoveringEyes),
                "hat_Cap" to resultCheckICAOResponseToMap(requirementsICAOResponse?.hat_Cap),
                "veilOverFace" to resultCheckICAOResponseToMap(requirementsICAOResponse?.veilOverFace),
                "mouthOpen" to resultCheckICAOResponseToMap(requirementsICAOResponse?.mouthOpen),
                "otherFaces" to resultCheckICAOResponseToMap(requirementsICAOResponse?.otherFaces)
            )
        }

        private fun resultCheckICAOResponseToMap(resultCheckICAOResponse: ResultCheckICAOResponse?): Map<String, Any?> {
            return hashMapOf(
                "isConformed" to resultCheckICAOResponse?.isConformed,
                "confidence" to resultCheckICAOResponse?.confidence
            )
        }

        private fun faceSpoofResponseToMap(faceSpoofResponse: FaceSpoofResponse?): Map<String, Any?> {
            return hashMapOf(
                "success" to faceSpoofResponse?.success,
                "result" to faceSpoofResponse?.result,
                "spoof" to faceSpoofResponse?.spoof
            )
        }
    }
}