import 'package:biopassid_flutter/biopassid_flutter.dart';
import 'package:biopassid_flutter_example/components/face_check.dart';
import 'package:biopassid_flutter_example/components/icao_result_bottom_sheet.dart';
import 'package:flutter/material.dart';

class HomeScreen extends StatefulWidget {
  const HomeScreen({Key? key}) : super(key: key);

  @override
  State<HomeScreen> createState() => _HomeScreenState();
}

class _HomeScreenState extends State<HomeScreen> {
  RequirementsICAOResponse? requirements;

  final biopassid = BioPassID();

  final config =
      BioPassIDConfigPreset.getConfig(BioPassIDConfigType.FACE_DETECTION)
          .setQualityKey('abcdef0123456789')
          .setIcao(true);

  void openCamera() async {
    await biopassid.buildCameraView(
      biopassidConfig: config,
      callback: handleBioPassIDEvent,
    );
  }

  void handleBioPassIDEvent(BioPassIDEvent event) {
    if (event.icao?.requirements != null) {
      showModal(event.icao!.requirements!);
    }
  }

  void showModal(RequirementsICAOResponse newRequirements) async {
    if (requirements == null) {
      setState(() => requirements = newRequirements);

      await showModalBottomSheet(
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.vertical(
            top: Radius.circular(28),
          ),
        ),
        isScrollControlled: true,
        context: context,
        builder: (context) => IcaoResultBottomSheet(
          icaoRequirementsResponse: requirements!,
        ),
      ).whenComplete(() {
        setState(() => requirements = null);
      });
    } else {
      setState(() => requirements = newRequirements);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FaceCheck(openCamera: openCamera);
  }
}
