import Flutter
import UIKit
import BioPassID
import SwiftUI
import Foundation

@available(iOS 13.0, *)
public class SwiftBiopassidFlutterPlugin: NSObject, FlutterPlugin, FlutterStreamHandler {
    
    private var eventSink: FlutterEventSink? = nil
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "biopassid_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftBiopassidFlutterPlugin()
        
        let event = FlutterEventChannel(name: "biopassid_flutter/event_channel", binaryMessenger: registrar.messenger())
        event.setStreamHandler(instance)

        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if (call.method == "getBioPassId") {
            let config = BioPassIdUtils.dictionaryToBioPassIdConfig(call: call)
            
            var rootViewController:UIViewController! = UIApplication.shared.keyWindow?.rootViewController
            
            let cameraView = CameraView(config: config, callback: { (event) -> Void in
                let bioPassIdEvent = BioPassIdUtils.bioPassIdEventToDictionary(event: event)
                
                if((self.eventSink) != nil) {
                    self.eventSink!(bioPassIdEvent)
                }
            })
            
            let contentView = UIHostingController(rootView: cameraView)
                
            rootViewController.present(contentView, animated: true, completion: nil)
            
            result(true)
            
        } else {
            result(FlutterMethodNotImplemented)
        }
    }
    
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        self.eventSink = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        self.eventSink = nil
        return nil
    }
}
