//
//  BioPassIdUtils.swift
//  biopassid_flutter
//
//  Created by Mauricio on 04/02/22.
//

import Foundation
import Flutter
import BioPassID
import AVFoundation

public class BioPassIdUtils {
    public static func dictionaryToBioPassIdConfig(call: FlutterMethodCall) -> BioPassIDConfig {
        
        
        let arguments = call.arguments as! NSDictionary
        
        let type = arguments["type"] as! String
        
        let biopassidConfig = arguments["biopassidConfig"] as! Dictionary<String, Any>
        
        
        let cameraPreset: AVCaptureSession.Preset = .vga640x480
        //        when (biopassidConfig!!["cameraPreset"] as String) {
        //            "LIVENESS" -> CameraPreset.LIVENESS
        //            else -> CameraPreset.FACE_FRONT
        //                }
        //
        let apiKey = biopassidConfig["apiKey"] as! Dictionary<String, String>
        
        let multibiometricsApiKey = apiKey["multibiometrics"] ?? ""
        let qualityApiKey = apiKey["quality"] ?? ""
        
        let defaultCameraPosition: AVCaptureDevice.Position
        let stringPosition = biopassidConfig["defaultCameraPosition"] as! String
        if (stringPosition == "FRONT") {
            defaultCameraPosition = .front
        } else {
            defaultCameraPosition = .back
        }
        
        let showCaptureButton = biopassidConfig["showCaptureButton"] as! Bool
        let showFlashButton = biopassidConfig["showFlashButton"] as! Bool
        let showFlipCameraButton =
        biopassidConfig["showFlipCameraButton"] as! Bool
        let flashEnabledByDefault =
        biopassidConfig["flashEnabledByDefault"] as! Bool
        // let actionDescription = biopassidConfig["actionDescription"] as! String
        let icao = biopassidConfig["icao"] as! Bool
        let spoofing = biopassidConfig["spoofing"] as! Bool
        // let showHelpText = biopassidConfig["showHelpText"] as! Bool
        
        let strings = biopassidConfig["strings"] as! Dictionary<String, String>
//        let chooseOtherButton = strings["chooseOtherButton"]!
//        let confirmButton = strings["confirmButton"]!
//        let sendPhoto = strings["sendPhoto"]!
        let helpText = strings["helpText"]!
        let screenTitle = strings["screenTitle"]!
        //         let newStrings = BioPassIDStrings(chooseOtherButton, confirmButton, sendPhoto, tooltip)
        
        let styles = biopassidConfig["styles"] as! Dictionary<String, String>
        // let captureButtonColor = styles["captureButtonColor"]!
        let faceColor = styles["faceColor"]!
        // let flipCameraColor = styles["flipCameraColor"]!
        let overlayColor = styles["overlayColor"]!
        let flashButtonEnabledColor = styles["flashButtonEnabledColor"]!
        //         let newStyles = BioPassIDStyles(captureButtonColor, faceColor, flipCameraColor)
        
        var config: BioPassIDConfig
        
        if (type == "LivenessDetectionBioPassIDConfig") {
            config = BioPassIDConfigPreset.getConfig(type: BioPassIDConfigType.LIVENESS_DETECTION)
        } else {
            config = BioPassIDConfigPreset.getConfig(type: BioPassIDConfigType.FACE_DETECTION)
        }
        
        
        
        config
            .setQualityKey(newValue: qualityApiKey)
            .setMultibiometricsKey(newValue: multibiometricsApiKey)
            .setDefaultCameraPosition(newValue: defaultCameraPosition)
            .setCameraPreset(newValue: cameraPreset)
            .setShowFlipCameraButton(newValue: showFlipCameraButton)
            .setFlashEnabledByDefault(newValue: flashEnabledByDefault)
            .setShowFlashButton(newValue: showFlashButton)
            .setShowCapturePhotoButton(newValue: showCaptureButton)
            .setICAO(newValue: icao)
            .setSpoofing(newValue: spoofing)

            // CONFIG STYLES
            .setStylesFaceColor(newValue: faceColor)
            .setStylesOverlayColor(newValue: overlayColor)
            // .setStylesFlipCameraColor(newValue: flipCameraColor)
            // .setStylesCaptureButtonColor(newValue: captureButtonColor)
            .setStylesFlashButtonEnabledColor(newValue: flashButtonEnabledColor)

            // CONFIG STRINGS
            // .setStringsConfirmButton(newValue: confirmButton)
            // .setStringsSendPhoto(newValue: sendPhoto)
            // .setStringsChooseOtherButton(newValue: chooseOtherButton)
            .setStringsHelpText(newValue: helpText)
            .setStringsScreenTitle(newValue: screenTitle)
        
            
        
        return config
    }
    
    public static func bioPassIdEventToDictionary(event: BioPassIDEvent) -> Dictionary<String, Any> {
        let icaoResponse: Dictionary<String, Any>?
        let spoofResponse: Dictionary<String, Any>?
        
        if let readICAO = event.icao?.ICAO {
            icaoResponse = icaoResponseToDictionary(icaoResponse: readICAO)
        } else {
            icaoResponse = nil
        }
        
        if let readSpoof = event.spoof {
            spoofResponse = faceSpoofResponseToDictionary(faceSpoofResponse: readSpoof)
        } else {
            spoofResponse = nil
        }
        
        return [
            "photo" : FlutterStandardTypedData(bytes: Data(event.photo)),
            "icao" : icaoResponse,
            "spoof" : spoofResponse
        ]
    }
    
    public static func icaoResponseToDictionary(icaoResponse: ICAOResponse) -> Dictionary<String, Any> {
        return [
            "faceImageConformed" : icaoResponse.FaceImageConformed,
            "isAllConformed" : icaoResponse.IsAllConformed,
            "requirements" : requirementsICAOResponseToDictionary(requirementsICAOResponse: icaoResponse.Requirements)
        ]
    }
    
    public static func requirementsICAOResponseToDictionary(requirementsICAOResponse: RequirementsICAOResponse) -> Dictionary<String, Any> {
        return [
            "blurred" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.Blurred),
            "lookingAway" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.LookingAway),
            "inkMarked" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.InkMarked),
            "unnaturalSkin" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.UnnaturalSkin),
            "tooDarkLight" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.TooDarkLight),
            "washedOut" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.WashedOut),
            "pixelation" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.Pixelation),
            "hairAcrossEyes" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.HairAcrossEyes),
            "eyesClosed" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.EyesClosed),
            "variedBackground" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.VariedBackground),
            "roll_Pitch_Yaw" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.Roll_Pitch_Yaw),
            "flashReflectionOnSkin" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.FlashReflectionOnSkin),
            "redEyes" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.RedEyes),
            "shadowsBehindHead" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.ShadowsBehindHead),
            "shadowsAcrossFace" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.ShadowsAcrossFace),
            "darkTintedLenses" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.DarkTintedLenses),
            "flashReflectionOnLenses" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.FlashReflectionOnLenses),
            "framesTooHeavy" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.FramesTooHeavy),
            "framesCoveringEyes" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.FramesCoveringEyes),
            "hat_Cap" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.Hat_Cap),
            "veilOverFace" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.VeilOverFace),
            "mouthOpen" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.MouthOpen),
            "otherFaces" : resultCheckICAOResponseToDictionary(resultCheckICAOResponse: requirementsICAOResponse.OtherFaces)
        ]
    }
    
    public static func resultCheckICAOResponseToDictionary(resultCheckICAOResponse: ResultCheckICAOResponse) -> Dictionary<String, Any> {
        return [
            "isConformed" : resultCheckICAOResponse.IsConformed,
            "confidence" : resultCheckICAOResponse.Confidence
        ]
    }
    
    public static func faceSpoofResponseToDictionary(faceSpoofResponse: FaceSpoofResponse) -> Dictionary<String, Any> {
        return [
            "success" : faceSpoofResponse.Success,
            "result" : faceSpoofResponse.result,
            "spoof" : faceSpoofResponse.spoof,
            "errorMessage": faceSpoofResponse.ErrorMessage
        ]
    }
}
