import 'dart:typed_data';

import 'package:biopassid_flutter/biopassid_flutter.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/biopassapi/icao/icao_response.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/biopassapi/spoof/face_spoof_response.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/configs/biopassid_config.dart';
import 'package:flutter/services.dart';

class BioPassID {
  final MethodChannel _methodChannel = const MethodChannel('biopassid_flutter');
  final EventChannel _eventChannel =
      const EventChannel('biopassid_flutter/event_channel');
  Stream<Map<Object?, Object?>> _bioPassIDEventStream = const Stream.empty();

  Future<void> buildCameraView({
    required BioPassIDConfig biopassidConfig,
    required Function(BioPassIDEvent) callback,
  }) async {
    try {
      await _methodChannel.invokeMethod(
        'getBioPassId',
        {
          'biopassidConfig': biopassidConfig.toMap(),
          'type': biopassidConfig.runtimeType.toString(),
        },
      );

      _receiveBioPassIDEventStream(callback);
    } on PlatformException catch (e) {
      print('PlatformException: ${e.toString()}');
    } catch (e) {
      print('Unknown error: ${e.toString()}');
    }
  }

  void _receiveBioPassIDEventStream(Function(BioPassIDEvent) callback) {
    try {
      _bioPassIDEventStream = _eventChannel
          .receiveBroadcastStream()
          .map<Map<Object?, Object?>>((value) => value);

      _bioPassIDEventStream.listen((event) {
        IcaoResponse? icao;
        FaceSpoofResponse? spoof;

        final photo = event['photo'] as Uint8List;

        if (event['icao'] != null) {
          final icaoMap =
              Map<String, dynamic>.from(event['icao'] as Map<Object?, Object?>);
          icao = IcaoResponse.fromMap(icaoMap);
        }

        if (event['spoof'] != null) {
          final spoofMap = Map<String, dynamic>.from(
              event['spoof'] as Map<Object?, Object?>);
          spoof = FaceSpoofResponse.fromMap(spoofMap);
        }

        final bioPassIDEvent = BioPassIDEvent(
          photo: photo,
          icao: icao,
          spoof: spoof,
        );

        callback(bioPassIDEvent);
      });
    } catch (e) {
      rethrow;
    }
  }
}
