import 'package:biopassid_flutter/src/biopassid_configurations/biopassid_strings.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/biopassid_styles.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/camera_lens_direction.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/camera_preset.dart';
import 'package:biopassid_flutter/src/biopassid_configurations/configs/biopassid_config.dart';
import 'package:flutter/material.dart';

class LivenessDetectionBioPassIDConfig implements BioPassIDConfig {
  final Map<String, String> _apiKey = {
    'multibiometrics': '',
    'quality': '',
  };
  CameraPreset _cameraPreset = CameraPreset.FACE_FRONT;
  CameraLensDirection _defaultCameraPosition = CameraLensDirection.FRONT;
  bool _showCaptureButton = false;
  bool _showFlashButton = false;
  bool _showFlipCameraButton = false;
  bool _flashEnabledByDefault = false;
  bool _showHelpText = true;
  bool _icao = false;
  bool _spoofing = false;
  BioPassIDStrings _strings = BioPassIDStrings();
  BioPassIDStyles _styles = BioPassIDStyles();

  @override
  Map<String, dynamic> toMap() {
    return {
      'apiKey': apiKey,
      'cameraPreset': cameraPreset.toShortString(),
      'defaultCameraPosition': defaultCameraPosition.toShortString(),
      'showCaptureButton': showCaptureButton,
      'showFlashButton': showFlashButton,
      'showFlipCameraButton': showFlipCameraButton,
      'flashEnabledByDefault': flashEnabledByDefault,
      'strings': strings.toMap(),
      'styles': styles.toMap(),
      'showHelpText': showHelpText,
      'icao': icao,
      'spoofing': spoofing,
    };
  }

  @override
  Map<String, String> get apiKey => _apiKey;

  @override
  BioPassIDConfig setMultibiometricsKey(String multibiometricsKey) {
    _apiKey['multibiometrics'] = multibiometricsKey;
    return this;
  }

  @override
  BioPassIDConfig setQualityKey(String qualityKey) {
    _apiKey['quality'] = qualityKey;
    return this;
  }

  @override
  CameraPreset get cameraPreset => _cameraPreset;

  @override
  BioPassIDConfig setCameraPreset(CameraPreset cameraPreset) {
    _cameraPreset = cameraPreset;
    return this;
  }

  @override
  CameraLensDirection get defaultCameraPosition => _defaultCameraPosition;

  @override
  BioPassIDConfig setDefaultCameraPosition(
      CameraLensDirection defaultCameraPosition) {
    _defaultCameraPosition = defaultCameraPosition;
    return this;
  }

  @override
  bool get showCaptureButton => _showCaptureButton;

  @override
  BioPassIDConfig setShowCaptureButton(bool showCaptureButton) {
    _showCaptureButton = showCaptureButton;
    return this;
  }

  @override
  bool get showFlashButton => _showFlashButton;

  @override
  BioPassIDConfig setShowFlashButton(bool showFlashButton) {
    _showFlashButton = showFlashButton;
    return this;
  }

  @override
  bool get showFlipCameraButton => _showFlipCameraButton;

  @override
  BioPassIDConfig setShowFlipCameraButton(bool showFlipCameraButton) {
    _showFlipCameraButton = showFlipCameraButton;
    return this;
  }

  @override
  bool get flashEnabledByDefault => _flashEnabledByDefault;

  @override
  BioPassIDConfig setFlashEnabledByDefault(bool flashEnabledByDefault) {
    _flashEnabledByDefault = flashEnabledByDefault;
    return this;
  }

  @override
  bool get icao => _icao;

  @override
  BioPassIDConfig setIcao(bool icao) {
    _icao = icao;
    return this;
  }

  @override
  bool get spoofing => _spoofing;

  @override
  BioPassIDConfig setSpoofing(bool spoofing) {
    _spoofing = spoofing;
    return this;
  }

  @override
  bool get showHelpText => _showHelpText;

  @override
  BioPassIDConfig setShowHelpText(bool showHelpText) {
    _showHelpText = showHelpText;
    return this;
  }

  // Strings
  @override
  BioPassIDStrings get strings => _strings;

  @override
  BioPassIDConfig setStrings(BioPassIDStrings strings) {
    _strings = strings;
    return this;
  }

  @override
  String get screenTitle => _strings.screenTitle;

  @override
  BioPassIDConfig setScreenTitle(String screenTitle) {
    _strings.screenTitle = screenTitle;
    return this;
  }

  @override
  String get helpText => _strings.helpText;

  @override
  BioPassIDConfig setHelpText(String helpText) {
    _strings.helpText = helpText;
    return this;
  }

  // Styles
  @override
  BioPassIDStyles get styles => _styles;

  @override
  BioPassIDConfig setStyles(BioPassIDStyles styles) {
    _styles = styles;
    return this;
  }

  @override
  Color get faceColor => _styles.faceColor;

  @override
  BioPassIDConfig setFaceColor(Color faceColor) {
    _styles.faceColor = faceColor;
    return this;
  }

  @override
  Color get overlayColor => _styles.overlayColor;

  @override
  BioPassIDConfig setOverlayColor(Color overlayColor) {
    _styles.overlayColor = overlayColor;
    return this;
  }

  @override
  Color get flashButtonEnabledColor => _styles.flashButtonEnabledColor;

  @override
  BioPassIDConfig setFlashButtonEnabledColor(Color flashButtonEnabledColor) {
    _styles.flashButtonEnabledColor = flashButtonEnabledColor;
    return this;
  }

  @override
  String toString() {
    return 'LivenessDetectionBioPassIDConfig(apiKey: $_apiKey, cameraPreset: $_cameraPreset, defaultCameraPosition: $_defaultCameraPosition, showCaptureButton: $_showCaptureButton, showFlashButton: $_showFlashButton, showFlipCameraButton: $_showFlipCameraButton, flashEnabledByDefault: $_flashEnabledByDefault, showHelpText: $_showHelpText, icao: $_icao, spoofing: $_spoofing)';
  }
}
